from mogi.models import CAnnotationMOGI

import django_filters

class CAnnotationMOGIFilter(django_filters.FilterSet):

    mzmed_gt = django_filters.NumberFilter(name='cannotation__cpeakgroup__mzmed', lookup_expr='gt', label='mzmed >')
    mzmed_lt = django_filters.NumberFilter(name='cannotation__cpeakgroup__mzmed', lookup_expr='lt', label='mzmed <')

    rtmed_gt = django_filters.NumberFilter(name='cannotation__cpeakgroup__rtmed', lookup_expr='gt', label='rtmed >')
    rtmed_lt = django_filters.NumberFilter(name='cannotation__cpeakgroup__rtmed', lookup_expr='lt',  label='rtmed <')

    compound_name = django_filters.CharFilter(name='cannotation__compound__name', lookup_expr='contains', label='compound name')

    class Meta:
        model = CAnnotationMOGI
        fields = ('mzmed_gt', 'mzmed_lt', 'rtmed_gt', 'rtmed_lt', 'compound_name')


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# -*- coding: utf-8 -*-

from django.shortcuts import render
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import View, ListView

from django_tables2.export.views import ExportMixin
from django_tables2.views import SingleTableMixin

from misa.models import Investigation
from misa.views import InvestigationListView
from misa.tables import ISAFileSelectTable, ISAFileSelectTableWithCheckBox
from misa.filter import ISAFileFilter

from mogi.tables import InvestigationTableUpload, WorkflowTableISA, HistoryMogiTable, HistoryMogiDataTable, CPeakGroupMetaMogiTable, CAnnotationMogiTable, IncomingGalaxyDataTable
from mogi.models import CAnnotationMOGI, CPeakGroupMetaMOGI, IncomingGalaxyData
from mogi.forms import ISAtoGalaxyParamForm, HistoryMogiDataForm, ISAWorkflowRunForm
from mogi.tasks import galaxy_isa_upload_datalib_task, save_lcms_mogi, download_cannotations_mogi_task
from mbrowse.utils.save_lcms import LcmsDataTransfer
from mbrowse.models import MFile, MetabInputData, CAnnotation
from mbrowse.views import CPeakGroupMetaListView, CAnnotationListAllView, CAnnotationDownloadView
from django.db.models import Q

from galaxy.models import Workflow, History
from galaxy.utils.history_actions import get_history_data,init_history_data_save_form, history_data_save_form
from galaxy.views import FilesToGalaxyDataLib, GenericFilesToGalaxyHistory, WorkflowListView, HistoryListView, HistoryDataCreateView
from galaxy.views import WorkflowRunView, TableFileSelectMixin
from galaxy.utils.history_actions import get_history_status
from mogi.models import HistoryDataMOGI
from mogi.filter import CAnnotationMOGIFilter

from rest_framework import viewsets
from mogi.serializers import IncomingGalaxyDataSerializer


########################################################################################################################
# REST
########################################################################################################################
class IncomingGalaxyDataViewSet(viewsets.ModelViewSet):
    """
    """
    queryset = IncomingGalaxyData.objects.all()
    serializer_class =  IncomingGalaxyDataSerializer


class IncomingGalaxyDataListView(LoginRequiredMixin, SingleTableMixin, ListView):
    '''
    '''
    table_class = IncomingGalaxyDataTable
    model = IncomingGalaxyData
    template_name = 'mogi/incoming_galaxy_data.html'




########################################################################################################################
# Galaxy ISA uploads and workflows
########################################################################################################################
class GalaxyISAupload(TableFileSelectMixin, InvestigationListView):
    '''
    '''
    success_msg = "Run started"
    # template_name = 'misa/investigation_list.html'
    table_class = InvestigationTableUpload
    form_class = ISAtoGalaxyParamForm
    template_name = 'mogi/isa_files_to_galaxy.html'
    initial_context = {'library': True, 'django_url': 'galaxy_isa_upload_datalib/'}

    def form_valid(self, request, form):
        user = self.request.user
        galaxy_isa_upload_param = form.save(commit=False)
        galaxy_isa_upload_param.added_by = user
        galaxy_isa_upload_param.save()

        pks = request.POST.getlist("check")

        result = galaxy_isa_upload_datalib_task.delay(pks, galaxy_isa_upload_param.id,
                                                      galaxy_pass=form.cleaned_data['galaxy_password'],
                                                      user_id=user.id)
        request.session['result'] = result.id
        return render(request, 'gfiles/status.html', {'s': 0, 'progress': 0})


class ISAWorkflowRunView(WorkflowRunView):
    '''
    Run a registered workflow
    '''
    success_msg = "Run started"
    success_url = '/galaxy/success'
    template_name = 'galaxy/workflow_run.html'
    table_class = ISAFileSelectTable
    filter_class = ISAFileFilter
    form_class = ISAWorkflowRunForm
    redirect_to = 'history_mogi'


class ISAFileSelectToGalaxyDataLib(FilesToGalaxyDataLib):
    table_class = ISAFileSelectTableWithCheckBox
    filterset_class = ISAFileFilter


class ISAFileSelectToGalaxyHist(GenericFilesToGalaxyHistory):
    table_class = ISAFileSelectTableWithCheckBox
    filterset_class = ISAFileFilter


class ISAWorkflowListView(WorkflowListView):
    table_class = WorkflowTableISA
    redirect_to = 'isa_workflow_summary'


########################################################################################################################
# Galaxy History data upload to django-metab
########################################################################################################################
class HistoryDataMogiListView(LoginRequiredMixin, View):

    def get(self, request, *args, **kwargs):
        data = get_history_data(self.kwargs['pk'], request.user, data_type=['sqlite'])

        table = HistoryMogiDataTable(data)

        return render(request, 'galaxy/history_data_bioblend_list.html', {'table': table})
        # return render(request, 'galaxy/history_status.html', {'table': table})

class HistoryMogiListView(HistoryListView):

    template_name = 'galaxy/history_status.html'
    table_class = HistoryMogiTable


class HistoryDataMogiCreateView(HistoryDataCreateView):
    model = HistoryDataMOGI
    form_class = HistoryMogiDataForm
    template_name = 'galaxy/historydata_form.html'

    def form_valid(self, form):
        obj = self.save_form(form)
        # first get all the mfiles associated with the investigation

        result = save_lcms_mogi.delay(obj.pk, self.request.user.id)
        self.request.session['result'] = result.id
        return render(self.request, 'gfiles/status.html', {'s': 0, 'progress': 0})


class HistoryDataMogiFromRestCreateView(HistoryDataMogiCreateView):

    def get_initial(self):
        user = self.request.user
        get_history_status(user)

        galaxy_name = self.kwargs.get('galaxy_name')
        galaxy_data_id = self.kwargs.get('galaxy_data_id')
        galaxy_history_id = self.kwargs.get('galaxy_history_id')

        internal_h = History.objects.filter(galaxy_id=galaxy_history_id, galaxyinstancetracking__name=galaxy_name)

        if internal_h:
            history_d = init_history_data_save_form(user=user, history_internal_id=internal_h[0].id, galaxy_dataset_id=galaxy_data_id)

            return {'history': internal_h[0].id,
                    'name': history_d['name']}
        else:
            return {'history': 'NO DATA AVAILABLE (PLEASE CHECK CONNECTION)',
                    'name': 'NO DATA AVAILABLE (PLEASE CHECK CONNECTION)'}

    def save_form(self, form):
        history_data_obj = form.save(commit=False)
        history_data_obj.user = self.request.user

        galaxy_name = self.kwargs.get('galaxy_name')
        galaxy_data_id = self.kwargs.get('galaxy_data_id')
        galaxy_history_id = self.kwargs.get('galaxy_history_id')

        internal_h = History.objects.filter(galaxy_id=galaxy_history_id, galaxyinstancetracking__name=galaxy_name)

        return history_data_save_form(self.request.user, internal_h[0].id, galaxy_data_id, history_data_obj)


class SaveLcmsFromFromRest(LoginRequiredMixin, View):


    def get(self, request, *args, **kwargs):
        return render(request, 'mogi/confirm_submission.html')

    def post(self, request, *args, **kwargs):
        galaxy_name = self.kwargs.get('galaxy_name')
        galaxy_data_id = self.kwargs.get('galaxy_data_id')
        galaxy_history_id = self.kwargs.get('galaxy_history_id')
        investigation_name = self.kwargs.get('investigation_name')

        result = save_lcms_mogi.delay(request.user.id,
                                      galaxy_name,
                                      galaxy_data_id,
                                      galaxy_history_id,
                                      investigation_name)

        self.request.session['result'] = result.id
        return render(self.request, 'gfiles/status.html', {'s': 0, 'progress': 0})







########################################################################################################################
# django-metab Peak and annotation summary
########################################################################################################################
class CPeakGroupMetaListMogiView(CPeakGroupMetaListView):
    table_class =CPeakGroupMetaMogiTable
    model = CPeakGroupMetaMOGI

    def get_queryset(self):
        # To get django-tables2 to play nicely with properties need tot trun queryset into list!
        # https://stackoverflow.com/questions/26985132/django-tables2-how-to-order-by-accessor
        qs = super(CPeakGroupMetaListMogiView, self).get_queryset()
        return list(qs)


class CAnnotationListAllMogiView(CAnnotationListAllView):
    table_class = CAnnotationMogiTable
    model = CAnnotationMOGI
    filterset_class = CAnnotationMOGIFilter
    template_name = 'mogi/cpeakgroup_annotations_all_mogi.html'

    def get_queryset(self):
        return self.model.objects.all().order_by('-cannotation__weighted_score')



class CAnnotationDownloadMogiView(CAnnotationDownloadView):

    def form_valid(self, form):
        obj = form.save()
        obj.user = self.request.user
        obj.save()

        result = download_cannotations_mogi_task.delay(obj.pk, self.request.user.id)
        self.request.session['result'] = result.id

        return render(self.request, 'gfiles/status.html', {'s': 0, 'progress': 0})


########################################################################################################################
# Generic
########################################################################################################################
def index(request):
    summary_d = {}
    summary_d['dataset_nm'] = len(MetabInputData.objects.all())

    summary_d['isa_nm'] = len(Investigation.objects.all())

    summary_d['workflow_nm'] = len(Workflow.objects.all())


    summary_d['ann_nm'] = len(CAnnotation.objects.filter(Q(spectral_matching_average_score__gt=0.6) |
                                                         Q(ms1_average_score__gt=0.0) |
                                                         Q(metfrag_average_score__gt=0.0) |
                                                         Q(sirius_csifingerid_average_score__gt=0.0) |
                                                         Q(mzcloud_average_score__gt=0.6)
                                                         ).values('cpeakgroup').distinct())


    return render(request, 'gfiles/index.html', summary_d)

def about(request):
    return render(request, 'mogi/about.html')

def submitted(request):
    return render(request, 'galaxy/submitted.html')

def success(request):
    return render(request, 'dma/success.html')








# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from django.contrib.auth.models import AnonymousUser, User
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import TestCase, RequestFactory, Client
from django.urls import reverse


from mogi.views import IncomingGalaxyDataListView

def add_middleware_to_request(request, middleware_class):
    middleware = middleware_class()
    middleware.process_request(request)
    return request

def add_middleware_to_response(request, middleware_class):
    middleware = middleware_class()
    middleware.process_response(request)
    return request




class IncomingGalaxyDataViewSetTestCase(TestCase):


    def setUp(self):

        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='jacob', email='jacob@jacob.com', password='top_secret')

    def test_login_redirect(self):
        """
        Test to check if a guest user is redirect to the login page
        """
        request = self.factory.get('/mogi/incoming_galaxy_data_list/')

        request.user = AnonymousUser()
        request = add_middleware_to_request(request, SessionMiddleware)
        response = IncomingGalaxyDataListView.as_view()(request)

        # client acts as a fake website for the request
        response.client = Client()

        self.assertRedirects(response, '/login/?next=/mogi/incoming_galaxy_data_list/')

    def test_get(self):
        """
        """
        request = self.factory.get(reverse('incoming_galaxy_data_list'))
        request.user = self.user
        response = IncomingGalaxyDataListView.as_view()(request)
        self.assertEqual(response.status_code, 200) # done without error


XXXXXXX XXXXXXX XX XXXXX XXXXXXXXXXXXX

XXXX XXXXX XXXXXXXXXXXXXXXXXXX
XXXXXX XXX XXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XX XXXXX XXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXXXXX
XX XXXXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXX XXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXX
XXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXX
XXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXX XX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXX XXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXXXX XXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XX XXXX XXXXXXXX XXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXX XXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX X XXXXXX XXXXXXXXXXXXX
XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXX XXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXX XXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXX XXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXX XXXXXXXXXXXX
XXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXX
XXX XXXXXX XXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXXX XXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXX XXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXXXXXXXX
XXX XXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXX XX XXXX XXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX X XXXXX XXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XX XXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXX XX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX X XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXX
XXXXX X XXXXXXX XX XXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XX XXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXX XXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXX XX XXXXXXXXX XXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX XXXX XXX XXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXXX XXXXXXXXXXXXXXXXXXX
XX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXX X XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXX XXXXXX XXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXX
XXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXXXX
XXXXX XXXX XXXXXXXXXXXXXXX
XXXX XXXX XXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXX XXXXX XXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XX XXXXXX XXXXXXXX XXXXXXXXXXXXXX
XXXXXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXX XXXXXXXXXXXXXXXXXX
XXXXX XXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXX XXXXXXXXXXXXXXXXXXXX
XX XXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XX XXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXX XXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXXXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXX XXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XX XXXXXX XXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXX XXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XX XXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXX XXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXX XXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXX XXXXXXXXXXXXXXXXXX
XXXXX XXXX XXXXXXXXXXXXXXXXXXXX
XXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XX XXX XXX XXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXX XXXXXXXXXXXXXXXXXXXXXX
XXXX XXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXX XX XXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXX XXXXXXXXXXXXXXXX
XXX XXXXX XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX XXXXXXXX XXXXXXXXXXXXXXXXXXX
XXXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXXX
XXX XXXXXXXX XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXX X XXXXXXXXXXXXX
XXXXXX XXXX XXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXX XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXX XXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXX XXXXXXXXXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXX XXXXXXXXXXXXXXXXXXXXX
XXXXX XXXXXXXXXXXXXX XXXXXXXXXXXXXXXXXXXX
XXXX XXXXXX XXXXXXXXXXXXXXXXX
XXXX XXXXXXX XXXXXXXXXXXXXXXXXXXXXXXXX
XXXX XXXXXXXXXXXXXXXX
XXXXXXXX XXXXXXXXXXXXXXXXXXXXXXXX



from sitetree.utils import tree, item

# Be sure you defined `sitetrees` in your module.
sitetrees = (
  # Standard tree for mogi tools
  tree('mogi_all', items=[
      # Then define items and their children with `item` function.
      item('MOGI', 'mogi.idetail', children=[
          item('Upload ISA project to Galaxy data library', 'galaxy_isa_upload_datalib', in_menu=True, in_sitetree=True),
          item('Upload ISA project to Galaxy history', 'galaxy_isa_upload_history', in_menu=True, in_sitetree=True),
          item('Upload ISA data files to Galaxy data library', 'galaxy_isa_fileselect_upload_datalib', in_menu=True, in_sitetree=True),
          item('Upload ISA data files to Galaxy history', 'galaxy_isa_fileselect_upload_history', in_menu=True, in_sitetree=True),
          item('View & Run Workflows', 'isa_workflow_summary', in_menu=True, in_sitetree=True),

      ])
  ]),
 # Trees for the whole MOGI project
 tree('dma', items=[
      # Then define items and their children with `item` function.
      item('DMA', 'init1', children=[
          item('DMA init project', 'init1', in_menu=True, in_sitetree=True),

      ])
  ]),

  tree('isa', items=[
      # Then define items and their children with `item` function.
      item('ISA', 'ilist', children=[
          item('View all ISA projects', 'ilist', in_menu=True, in_sitetree=True),
          item('Create investigation', 'icreate', in_menu=True, in_sitetree=True),
          item('Create study', 'screate', in_menu=True, in_sitetree=True),
          item('Create assay', 'acreate', in_menu=True, in_sitetree=True),
          item('Create study sample', 'smcreate', in_menu=True, in_sitetree=True),
          item('Create study factor', 'sfcreate', in_menu=True, in_sitetree=True),
          item('View ISA data files', 'view_isa_data_files', in_menu=True, in_sitetree=True),
      ])
  ]),

  tree('galaxy', items=[
      # Then define items and their children with `item` function.
      item('Galaxy', 'history_status', children=[

          item('Register Galaxy instance', 'addgi', in_menu=True, in_sitetree=True),
          item('Register Galaxy user', 'addguser', in_menu=True, in_sitetree=True),
          item('Upload ISA project to Galaxy data library', 'galaxy_isa_upload_datalib', in_menu=True, in_sitetree=True),
          item('Upload ISA data files to Galaxy data library', 'galaxy_isa_fileselect_upload_datalib', in_menu=True, in_sitetree=True),
          item('View & Run Workflows', 'isa_workflow_summary', in_menu=True, in_sitetree=True),
          item('History status', 'history_mogi', in_menu=True, in_sitetree=True),

      ])
  ]),


  tree('view', items=[
      # Then define items and their children with `item` function.
      item('View', 'cpeakgroupmeta_summary', children=[
          item('Show all metabolomics files', 'mfile_summary', in_menu=True, in_sitetree=True),
          item('Show LC-MS datasets', 'cpeakgroupmeta_summary', in_menu=True, in_sitetree=True)
      ])
  ]),

  tree('search', items=[
      # Then define items and their children with `item` function.
      item('Search', 'init1', children=[
          item('Summary', 'search_summary',  in_menu=True, in_sitetree=True),
          item('Fragmentation search', 'search_frag',  in_menu=True, in_sitetree=True),
          item('Search m/z', 'search_mz',  in_menu=True, in_sitetree=True),
          item('Search neutral mass', 'search_nm',  in_menu=True, in_sitetree=True),
      ])
  ])

  # ... You can define more than one tree for your app.
)


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-08-28 05:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('galaxy', '0001_initial'),
        ('mbrowse', '0002_CUSTOM_IMPORT_DATA'),
        ('misa', '0002_IMPORT_DATA'),
    ]

    operations = [
        migrations.CreateModel(
            name='AnnotationSummary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lcms_ann_level', models.TextField(blank=True, max_length=100, null=True)),
                ('best_score', models.FloatField(blank=True, null=True)),
                ('assays', models.TextField(blank=True, max_length=100, null=True)),
                ('mzmin', models.FloatField(blank=True, null=True)),
                ('mzmax', models.FloatField(blank=True, max_length=100, null=True)),
                ('rtmin', models.FloatField(blank=True, max_length=100, null=True)),
                ('rtmax', models.FloatField(blank=True, null=True)),
                ('compound', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mbrowse.Compound')),
            ],
        ),
        migrations.CreateModel(
            name='CAnnotationMOGI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cannotation', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='mbrowse.CAnnotation')),
            ],
        ),
        migrations.CreateModel(
            name='CPeakGroupMetaMOGI',
            fields=[
                ('cpeakgroupmeta_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mbrowse.CPeakGroupMeta')),
                ('assay', models.ManyToManyField(to='misa.Assay')),
            ],
            bases=('mbrowse.cpeakgroupmeta',),
        ),
        migrations.CreateModel(
            name='HistoryDataMOGI',
            fields=[
                ('historydata_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='galaxy.HistoryData')),
                ('investigation', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='misa.Investigation')),
            ],
            bases=('galaxy.historydata',),
        ),
        migrations.CreateModel(
            name='IncomingGalaxyData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('galaxy_url', models.TextField(blank=True, max_length=100, null=True)),
                ('galaxy_name', models.TextField(max_length=100)),
                ('galaxy_data_id', models.TextField(max_length=100)),
                ('galaxy_history_id', models.TextField(max_length=100)),
                ('galaxy_history_name', models.TextField(blank=True, max_length=100, null=True)),
                ('other_details', models.TextField(blank=True, max_length=100, null=True)),
                ('investigation_name', models.TextField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='ISAGalaxyTrack',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('galaxy_id', models.CharField(max_length=250)),
                ('galaxyinstancetracking', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='galaxy.GalaxyInstanceTracking')),
                ('investigation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='misa.Investigation')),
                ('isatogalaxyparam', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='galaxy.FilesToGalaxyDataLibraryParam')),
            ],
        ),
        migrations.AddField(
            model_name='cpeakgroupmetamogi',
            name='historydatamogi',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mogi.HistoryDataMOGI'),
        ),
    ]



from django.contrib.auth.models import User

from mbrowse.utils.save_lcms import LcmsDataTransfer
from mogi.models import CPeakGroupMetaMOGI, CAnnotationMOGI, HistoryDataMOGI
from misa.models import Assay, AssayRun, AssayDetail, Investigation
from galaxy.models import History

from mogi.forms import HistoryMogiDataForm
from galaxy.utils.history_actions import get_history_status, history_data_save_form, init_history_data_save_form

class LcmsDataTransferMOGI(LcmsDataTransfer):
    cpeakgroupmeta_class = CPeakGroupMetaMOGI
    historydatamogi = ''
    assays = []

    def set_cpeakgroupmeta(self, celery_obj):

        cpgm = CPeakGroupMetaMOGI(metabinputdata=self.md, historydatamogi=self.historydatamogi)
        cpgm.save()

        ########################################
        # Check ISA has been created for files
        ########################################
        # if the files are not assigned to an assay we can't process properly as we can not reference to the correct
        # study sample
        for mfile in self.mfile_d.values():
            if not mfile.run.assayrun:
                if celery_obj:
                    celery_obj.update_state(state='FAILED',
                                                meta={'current': 0, 'total': 100,
                                                      'status': 'Data file {} used is not associated with any '
                                                                'Assay '.format(mfile.original_filename)})

                return 0

        ###########################################
        # Get relevant assays
        ###########################################

        assays = [Assay.objects.get(id=mfile.run.assayrun.assaydetail.assay_id) for mfile in set(self.mfile_d.values())]
        assays = list(set(assays))

        md_name = '|'

        for a in assays:
            md_name += '{} {} {}|'.format(a.study.investigation_id, a.study_id, a.name)
            cpgm.assay.add(a)

        self.md.name = md_name
        self.md.save()
        self.assays = assays
        self.cpgm = cpgm

        return cpgm


def get_data_from_galaxy(user_id, galaxy_name, galaxy_data_id, galaxy_history_id, investigation_name, celery_obj):
    if celery_obj:
        celery_obj.update_state(state='RUNNING',
                                meta={'current': 0.1, 'total': 100, 'status': 'Getting data from Galaxy instance'})

    user = User.objects.get(pk=user_id)

    get_history_status(user)

    internal_h = History.objects.filter(galaxy_id=galaxy_history_id, galaxyinstancetracking__name=galaxy_name)

    if not internal_h:
        error_msg = 'No data available please check galaxy connection'
        print(error_msg)
        if celery_obj:
            celery_obj.update_state(state='FAILED', meta={'current': 0, 'total': 100, 'status': error_msg})
        return 0, error_msg

    i_qs = Investigation.objects.filter(slug=investigation_name)

    if not i_qs:
        error_msg = 'No investigation with name {}'.format(investigation_name)
        print(error_msg)
        if celery_obj:
            celery_obj.update_state(state='FAILED', meta={'current': 0, 'total': 100, 'status': error_msg})
        return 0, error_msg

    hdm = HistoryDataMOGI(investigation = i_qs[0], history=internal_h[0])

    print(hdm)
    hdm = history_data_save_form(user, internal_h[0].id, galaxy_data_id, hdm)

    return hdm



# Create your tasks here
from __future__ import absolute_import, unicode_literals
from celery import shared_task

from mogi.utils.upload_isa_to_galaxy import galaxy_isa_upload_datalib
from galaxy.models import FilesToGalaxyDataLibraryParam
from mbrowse.models import MFile, MetabInputData, CAnnotation
from gfiles.models import TrackTasks
from django.urls import reverse
from mbrowse.utils.download_annotations import DownloadAnnotations

from mogi.utils.save_lcms import LcmsDataTransferMOGI, get_data_from_galaxy

from mogi.models import HistoryDataMOGI, CAnnotationMOGI
from mogi.tables import CAnnotationMogiTable

@shared_task(bind=True)
def galaxy_isa_upload_datalib_task(self, pks, param_pk, galaxy_pass, user_id):
    """

    """
    tt = TrackTasks(taskid=self.request.id, state='RUNNING', name='Uploading ISA projects to Galaxy', user_id=user_id)
    tt.save()

    self.update_state(state='RUNNING', meta={'current': 0.1, 'total': 100, 'status': 'Uploading ISA projects to Galaxy'})
    galaxy_isa_upload_param = FilesToGalaxyDataLibraryParam.objects.get(pk=param_pk)

    result_out = galaxy_isa_upload_datalib(pks, galaxy_isa_upload_param, galaxy_pass, user_id, self)

    if result_out:
        self.update_state(state='SUCCESS', meta={'current': 100, 'total': 100, 'status': 'completed'})

    # save successful result
    tt.result = reverse('galaxy_summary')
    tt.state = 'SUCCESS'
    tt.save()



@shared_task(bind=True)
def save_lcms_mogi(self, userid, galaxy_name, galaxy_data_id, galaxy_history_id, investigation_name):
    ###################
    # Save task
    ###################
    tt = TrackTasks(taskid=self.request.id, state='RUNNING', name='LC-MSMS data upload', user_id=userid)
    tt.save()

    ####################
    # Perform operation
    ####################
    hdm = get_data_from_galaxy(userid, galaxy_name, galaxy_data_id, galaxy_history_id, investigation_name, self)

    if not hdm:
        tt.result = reverse('cpeakgroupmeta_summary_mogi')
        tt.state = 'FAILURE'
        tt.save()
        return 0

    mfiles = MFile.objects.filter(run__assayrun__assaydetail__assay__study__investigation=hdm.investigation.pk)
    mfiles_ids = [m.id for m in mfiles]
    md = MetabInputData(gfile_id=hdm.genericfile_ptr_id)
    md.save()
    lcms_data_transfer = LcmsDataTransferMOGI(md.id, mfiles_ids)
    lcms_data_transfer.historydatamogi = hdm

    if not lcms_data_transfer.transfer(celery_obj=self):
        # something wen't wrong don't perform the remaining analysis
        return 0

    self.update_state(state='RUNNING', meta={'current': 98, 'total': 100, 'status': 'Updating ISA links '})
    cans_mogi = []
    for i, cann in enumerate(CAnnotation.objects.filter(cpeakgroup__cpeakgroupmeta__metabinputdata=md).all()):
        if i % 5000 == 0:

            CAnnotationMOGI.objects.bulk_create(cans_mogi)
            cans_mogi = []
        cans_mogi.append(CAnnotationMOGI(cannotation=cann))

    CAnnotationMOGI.objects.bulk_create(cans_mogi)


    ####################
    # Save data
    ####################
    tt.result = reverse('cpeakgroupmeta_summary_mogi')
    tt.state = 'SUCCESS'
    tt.save()


class DownloadAnnotationsMogi(DownloadAnnotations):
    annotation_model_class = CAnnotationMOGI
    annotation_table_class = CAnnotationMogiTable

    def get_items(self, cann_down):
        if cann_down.rank:
            canns = self.annotation_model_class.objects.filter(cannotation__rank__lte=cann_down.rank)
        else:
            canns = self.annotation_model_class.objects.all()

        return canns


@shared_task(bind=True)
def download_cannotations_mogi_task(self, pk, userid):
    # track tasks in db
    tt = TrackTasks(taskid=self.request.id,
                    state='RUNNING',
                    name='Downloading LC-MSMS annotations',
                    user_id=userid)
    tt.save()

    # perform function
    dam = DownloadAnnotationsMogi()
    dam.download_cannotations(pk, self)


    # save successful result
    tt.result = reverse('canns_download_result')
    tt.state = 'SUCCESS'
    tt.save()









from misa.tables import InvestigationTable
import django_tables2 as tables
from django_tables2.utils import A
from misa.models import Investigation
from galaxy.models import Workflow, History
from galaxy.tables import HistoryTable, HistoryDataTable
from mbrowse.tables import CAnnotationTable, CPeakGroupMetaTable, NumberColumn2, NumberColumn4
from mogi.models import CPeakGroupMetaMOGI, CAnnotationMOGI, IncomingGalaxyData
from gfiles.utils.icons import EYE, DOWN, PLAY, SAVE
from mbrowse.models import CAnnotation
from django_tables2_column_shifter.tables import ColumnShiftTable
from django.utils.text import slugify

TABLE_CLASS = "mogi table-bordered table-striped table-condensed table-hover"


class InvestigationTableUpload(ColumnShiftTable):
    details = tables.LinkColumn('idetail', text=EYE, args=[A('id')])

    check = tables.CheckBoxColumn(accessor="id",
                                               attrs={
                                                   "th__input": {"onclick": "toggle(this)"},
                                                   "td__input": {"onclick": "addfile(this)"}},
                                               )

    class Meta:
        model = Investigation

        attrs = {"class": TABLE_CLASS}
        fields = ('id','name','description', 'details')




class WorkflowTableISA(ColumnShiftTable):
    run = tables.LinkColumn('isa_workflow_run', text=PLAY, args=[A('id')])

    def get_column_default_show(self):
        self.column_default_show = ['id', 'name', 'galaxyinstancetracking', 'accessible', 'run']
        return super(WorkflowTableISA, self).get_column_default_show()

    class Meta:

        model = Workflow
        attrs = {"class": TABLE_CLASS}



class HistoryMogiTable(HistoryTable):
    history_mogi_data = tables.LinkColumn('history_mogi_data', text=EYE, args=[A('id')])

    def get_column_default_show(self):
        self.column_default_show = ['galaxyinstancetracking', 'name', 'update_time', 'running', 'estimated_progress', 'history_mogi_data', 'check']
        return super(HistoryTable, self).get_column_default_show()

    class Meta:
        model = History
        sequence = ( 'id', 'galaxyinstancetracking', 'name', 'update_time', 'empty', 'failed_metadata', 'new', 'ok', 'paused', 'error',
                     'queued', 'setting_metadata', 'upload', 'running', 'estimated_progress', 'galaxy_id', 'history_data_bioblend_list',
                     'history_mogi_data', 'check')
        attrs = {"class": TABLE_CLASS}
        # running_tasks_details = tables.Column()
        order_by = ('-update_time',)






class HistoryMogiDataTable(HistoryDataTable):
    mogi_create = tables.LinkColumn('history_mogi_data_save', verbose_name='Save metabolomics data',
                                            text=SAVE, args=[A('history_internal_id'), A('id')])



    def get_column_default_show(self):
        self.column_default_show = ['galaxy_instance', 'name', 'data_type', 'create_time', 'download_url', 'mogi_create']
        return super(HistoryDataTable, self).get_column_default_show()

    class Meta:
        attrs = {"class": TABLE_CLASS}

#
class CPeakGroupMetaMogiTable(ColumnShiftTable):



    filename = tables.Column(accessor='filename', verbose_name='Input Filename', orderable=True, attrs={'test':'test'})
    investigation = tables.Column(accessor='historydatamogi.investigation.name', verbose_name='Investigation')
    study = tables.Column(accessor='study_names', verbose_name='Study')
    assay = tables.Column(accessor='assay_names')
    c_peak_group_table = tables.LinkColumn('cpeakgroup_summary', verbose_name='View grouped peaklist', text=EYE, args=[A('id')])

    class Meta:
        model = CPeakGroupMetaMOGI
        attrs = {"class": TABLE_CLASS}
        fields = ('id', 'date', 'metabinputdata', 'filename', 'investigation', 'study', 'assay', 'polarity', 'c_peak_group_table')

        template = 'django_tables2/bootstrap.html'


# class CPeakGroupMetaMogiTable(ColumnShiftTable):
#     # galaxy_history = tables.Column(accessor='historydatamogi', verbose_name='Galaxy history details')
#
#     # galaxy_history = tables.Column(empty_values=())
#
#     # investigation = tables.Column(accessor='historydatamogi.investigation',
#     #                                verbose_name='Investigation')
#
#
#     class Meta:
#         model = ISAGalaxyTrack
#         attrs = {'class': 'paleblue'}
#         template = 'django_tables2/bootstrap.html'







class CAnnotationMogiTable(ColumnShiftTable):
    # galaxy_history = tables.Column(accessor='cpeakgroup.cpeakgroupmeta.metabinputdata.historymogi.name', verbose_name='Galaxy history details')
    #
    inputdata = tables.Column(accessor='cannotation.cpeakgroup.cpeakgroupmeta.metabinputdata.id',
                              verbose_name='Input Dataset (id)')

    inputdata_name = tables.Column(accessor='cannotation.cpeakgroup.cpeakgroupmeta.metabinputdata.name',
                              verbose_name='Input Dataset (name)')

    investigation = tables.Column(accessor='investigation_names', verbose_name='Investigation')
    study = tables.Column(accessor='study_names',
                                   verbose_name='Study')

    assay = tables.Column(accessor='assay_names',
                                   verbose_name='Assay')

    polarity = tables.Column(accessor='cannotation.cpeakgroup.cpeakgroupmeta.polarity', verbose_name='Polarity')

    galaxy_history_name = tables.Column(accessor='galaxy_history_name', verbose_name='Galaxy history)')
    galaxy_history_data_name = tables.Column(accessor='galaxy_history_data_name', verbose_name='Galaxy history (data)')

    inchikey = tables.Column(accessor='cannotation.compound.inchikey_id', verbose_name='InChIKey')
    compound_name = tables.Column(accessor='cannotation.compound.name', verbose_name='Compound name')
    pubchem_ids = tables.Column(accessor='cannotation.compound.pubchem_id', verbose_name='PubChem cid(s)')
    kegg_ids = tables.Column(accessor='cannotation.compound.kegg_id', verbose_name='KEGG cid(s)')

    # library_spectra = tables.Column(accessor='cannotation.compound.library_spectra_meta.name',
    #                                 verbose_name='Library spectra')
    # library_spectra_accession = tables.Column(accessor='cannotation.compound.library_spectra_meta.name',
    #                                 verbose_name='Library spectra accession')


    compound_id =  tables.Column(accessor='cannotation.cpeakgroup.id', verbose_name='C Peak Group ID')
    mzmed = NumberColumn4(accessor='cannotation.cpeakgroup.mzmed',verbose_name='MZ Med')
    rtmed = NumberColumn2(accessor='cannotation.cpeakgroup.rtmed', verbose_name='RT Med')

    spectral_matching_average_score = NumberColumn2(accessor='cannotation.spectral_matching_average_score')
    metfrag_average_score = NumberColumn2(accessor='cannotation.metfrag_average_score')
    mzcloud_average_score = NumberColumn2(accessor='cannotation.mzcloud_average_score')
    sirius_csifingerid_average_score = NumberColumn2(accessor='cannotation.sirius_csifingerid_average_score')
    ms1_average_score = NumberColumn2(accessor='cannotation.ms1_average_score')
    weighted_score = NumberColumn2(accessor='cannotation.weighted_score')
    rank = tables.Column(accessor='cannotation.rank')



    # how to get assay? Details are stored in the filenames... Perhaps better to create a new model to store this information

    # def get_column_default_show(self):
    #     self.column_default_show = ['galaxy_instance', 'name', 'data_type', 'create_time', 'download_url', 'mogi_create']
    #     return super(HistoryDataTable, self).get_column_default_show()
    #
    class Meta:
        attrs = {"class": TABLE_CLASS}

        model = CAnnotationMOGI


class IncomingGalaxyDataTable(ColumnShiftTable):
    mogi_create = tables.LinkColumn('save_lcms_from_from_rest', verbose_name='Save metabolomics data',
                                            text=SAVE, args=[A('galaxy_name'),
                                                             A('galaxy_data_id'),
                                                             A('galaxy_history_id'),
                                                             A('investigation_name')]
                                     )

    class Meta:
        attrs = {"class": TABLE_CLASS}
        model = IncomingGalaxyData

#


from rest_framework import serializers
from mogi.models import IncomingGalaxyData


class IncomingGalaxyDataSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = IncomingGalaxyData
        fields = ('__all__')


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# from django.contrib.auth.models import AnonymousUser, User
# from django.contrib.sessions.middleware import SessionMiddleware
# from django.test import TestCase, RequestFactory, Client



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# general python

# standard django
from django import forms

# django external apps
from dal import autocomplete

# django custom user external apps
from galaxy.forms import FilesToGalaxyDataLibraryParamForm, WorkflowRunForm
from galaxy.models import FilesToGalaxyDataLibraryParam
from mogi.models import HistoryDataMOGI


class ISAtoGalaxyParamForm(FilesToGalaxyDataLibraryParamForm):

    class Meta:
        model = FilesToGalaxyDataLibraryParam
        fields = ('galaxyinstancetracking', 'remove', 'link2files', 'ftp')
        widgets = {
            'galaxyinstancetracking': autocomplete.ModelSelect2(url='galaxyinstancetracking-autocomplete'),
        }


class ISAWorkflowRunForm(WorkflowRunForm):
    auto_samplelist = forms.BooleanField(required=False)




class HistoryMogiDataForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(HistoryMogiDataForm, self).__init__(*args, **kwargs)
        self.fields['history'].disabled = True
        self.fields['name'].disabled = True

    class Meta:
        model = HistoryDataMOGI
        fields = ('history', 'name', 'investigation')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os

from django.db import models
from django.utils.functional import cached_property

from galaxy.models import GalaxyInstanceTracking, FilesToGalaxyDataLibraryParam, HistoryData
from misa.models import Investigation, Assay, Study

from mbrowse.models import CAnnotation, Compound, CPeakGroupMeta


################################################################################################################
# ISA - Galaxy interfacing
################################################################################################################
class ISAGalaxyTrack(models.Model):
    galaxyinstancetracking = models.ForeignKey(GalaxyInstanceTracking, on_delete=models.CASCADE, null=False,
                                               blank=False)
    isatogalaxyparam = models.ForeignKey(FilesToGalaxyDataLibraryParam, on_delete=models.CASCADE)
    investigation = models.ForeignKey(Investigation, on_delete=models.CASCADE)
    galaxy_id = models.CharField(max_length=250)

    def __str__(self):              # __unicode__ on Python 2
        return self.galaxy_id

class HistoryDataMOGI(HistoryData):
    investigation = models.ForeignKey(Investigation, on_delete=models.CASCADE, null=True, blank=True)

    def __str__(self):              # __unicode__ on Python 2
        return self.investigation.name




class CPeakGroupMetaMOGI(CPeakGroupMeta):
    historydatamogi = models.ForeignKey(HistoryDataMOGI, on_delete=models.CASCADE)
    assay = models.ManyToManyField(Assay)

    @cached_property
    def assay_names(self):
        return ', '.join(l.name for l in self.assay.all())


    @cached_property
    def study_names(self):
        return ', '.join(l['study__name'] for l in self.assay.all().values('study__name'))

    @cached_property
    def filename(self):
        return os.path.basename(self.metabinputdata.gfile.data_file.name)

    @cached_property
    def filepath(self):
        return self.metabinputdata.gfile.data_file






class CAnnotationMOGI(models.Model):
    cannotation = models.OneToOneField(CAnnotation)
    # investigation_names = models.CharField(max_length=100, null=True, blank=True)

    # originally as properties but took too long calculate. Quicker (to save at the point of upload)
    @property
    def investigation_names(self):
        m = self.cannotation.cpeakgroup.cpeakgroupmeta.metabinputdata
        hdm = HistoryDataMOGI.objects.get(metabinputdata=m)
        return hdm.investigation.name

    @property
    def study_names(self):
        cpg = self.cannotation.cpeakgroup

        return ', '.join(a.name for a in Study.objects.filter(assay__assaydetail__assayrun__run__mfile__xcmsfileinfo__cpeak__cpeakgroup=cpg).distinct())

    @property
    def assay_names(self):
        cpg = self.cannotation.cpeakgroup
        return ', '.join(a.name for a in Assay.objects.filter(assaydetail__assayrun__run__mfile__xcmsfileinfo__cpeak__cpeakgroup=cpg).distinct())

    @property
    def galaxy_history_data_name(self):
        # for mogi only
        m = self.cannotation.cpeakgroup.cpeakgroupmeta.metabinputdata
        hdm = HistoryDataMOGI.objects.get(metabinputdata=m)
        return hdm.name

    @property
    def galaxy_history_name(self):
        # for mogi only
        m = self.cannotation.cpeakgroup.cpeakgroupmeta.metabinputdata
        hdm = HistoryDataMOGI.objects.get(metabinputdata=m)
        return hdm.history.name




class AnnotationSummary(models.Model):
    compound = models.ForeignKey(Compound, on_delete=models.CASCADE)
    lcms_ann_level = models.TextField(max_length=100, null=True, blank=True)
    best_score = models.FloatField(null=True, blank=True)
    assays = models.TextField(max_length=100, null=True, blank=True)
    mzmin = models.FloatField(null=True, blank=True)
    mzmax = models.FloatField(max_length=100, null=True, blank=True)

    rtmin = models.FloatField(max_length=100, null=True, blank=True)
    rtmax = models.FloatField(null=True, blank=True)

    def __str__(self):              # __unicode__ on Python 2
        return '{} {}'.format(self.compound, self.lcms_ann_level)



class IncomingGalaxyData(models.Model):
    galaxy_url = models.TextField(max_length=100, null=True, blank=True)
    galaxy_name = models.TextField(max_length=100)
    galaxy_data_id = models.TextField(max_length=100)
    galaxy_history_id = models.TextField(max_length=100)
    galaxy_history_name = models.TextField(max_length=100, null=True, blank=True)
    other_details = models.TextField(max_length=100, null=True, blank=True)
    investigation_name = models.TextField(max_length=100)

    def __str__(self):              # __unicode__ on Python 2
        return self.galaxy_data_id



XXXXXXXXX XXXXXX XXXXXXXXXX XXX XXXXX XXXXXXXXXXXXX XXXXXXXXXXXXXXXXXXX

XXXX XXXXXXXX XXXXXXXX XX XXXXXXXXX XXXXXXXXXXXXX XXXX XX XXXX
XXXXXXXXXXXX XXX XXXXX XXXXXXXXXXXX XXXXXXXX XXX XXX XXXXXXXX XXXXXXX
XXXXXXXXX XX XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

XXX XXXXXXXXX XXXXXXX XXXXXXX XX XXX XXXXX XX XXXX XXXXXXXX XXXXXX XX
XXXXXXXXXX XXXXXX

XXXX

XXXXXXXXXX XX XXXXXX XXXXXXXX XXXX XX XXXXXXX XX XXX XXXXXX XXXXXXXXX
X XXXX XX XXXX XXXXXXXX XXX XXXXXXXXXX XXXXXXXXXXXXX XXXXX XXXX
XXXXXXXXXXXX XX XXXX XX XXX XXXXXXXX XXXXXXX XXXXXXXXXXXX XXXXXXXXX
XXXXXXX XXXXXXXXXX XXX XXXXXX XX XXXX XXXXX XXXXXXX XXXXXX XXXXXXXX
XXXXXXXXXXX XXXXXXXXXXX XXXXXX XXXX XXXXXX XX XXX XXXXXXXXX XXX XX
XXXXXX XXXXXXX XX XXXX XXX XXXXXXXX XX XXXXXXXXX XX XX XXX XXXXXXX XX
XXX XXXXXXXXX XXXXXXXXXXX

XXX XXXXX XXXXXXXXX XXXXXX XXX XXXX XXXXXXXXXX XXXXXX XXXXX XX
XXXXXXXX XX XXX XXXXXX XX XXXXXXXXXXX XXXXXXXX XX XXX XXXXXXXXX

XXX XXXXXXXX XX XXXXXXXX XXX XXXX XXXXXXX XXXXXXXX XX XXX XXXXX
XXXXXXX XX XXXXXXXX XXXXXXXXX XXX XXX XXXXXXX XX XXX XXXXXXXXXX XX
XXXXXXXXXXXXXXXX XXXXXXX XXX X XXXXXXXXXX XXXXXXX XXX
XXXXXXXXXXXXXXXX XX XX XXXXX XXXXX XXX XXXXXXX XX XXXXXXXXX XXXXXXX XX
XXXXXX XXX XXX XXXXXX XXXXXXX XX XXXXX XXXXXXXXXX XXXXXXX XX XX XXXXXX
XX XXXXXXXXX XXXX XX XXXXXXXXXX XXXXXXX XXXXX XXX XX XX XX XXXXXXXXXX
XXXX XXX XXXXXXXX XX XXX XXX XX XXXXX XXXXXXXX XX XXX XXXXXXXXX

XXXX

XXX XXXXX XXXXXXX XX XXX XXXXXXXXXXXX XXX XXXXXXXX XXXXXXXXXXX XXX
XXXXXXXXXX XXXXXXXXXX XXXXXXXXX XXXX XX XXXX XXXXXXXX XXXXX XXXX XXXXX
XXX XXXXXXXXX XX XXXXXXXXX XXX XXXX XXXXX XX XXXXX XXXXX XXX XXXXXX XXXX
XXX XXXXX XXXXXX



from __future__ import print_function
import itertools
import os
import tempfile
import shutil
import csv
import sys
from operator import itemgetter
from django.core.files import File
from bioblend.galaxy.libraries import LibraryClient
from bioblend.galaxy.folders import FoldersClient
from bioblend.galaxy.client import ConnectionError
from io import TextIOWrapper

from django.conf import settings

from galaxy.utils.galaxy_utils import create_library, get_gi_gu
from galaxy.utils.upload_to_galaxy import add_filelist_datalib, link_files_in_galaxy
from misa.models import Investigation, MISAFile
from mbrowse.models import MFile
from ftplib import FTP, error_perm

from mogi.models import ISAGalaxyTrack


def galaxy_isa_upload_datalib(pks, galaxy_isa_upload_param, galaxy_pass, user_id, celery_obj=''):

    # ... Should this just be for admin? or shall all user have ability ? .... not sure

    # update celery
    if celery_obj:
        celery_obj.update_state(state='RUNNING',
                          meta={'current': 0.1, 'total': 100, 'status': 'Initialising galaxy'})

    # get the galaxy clients required for updating the galaxy instance
    git = galaxy_isa_upload_param.galaxyinstancetracking
    gi, gu = get_gi_gu(galaxy_isa_upload_param.added_by, git)
    lc = LibraryClient(gi)

    # Retrieve or create the base library used for all ISA folders
    lib = create_library(lc, 'mogi')

    # get all associated files for the selected ISA projects
    mfiles = get_mfile_qs(pks)
    #
    # # Add the files to Galaxy data library

    try:
        create_isa_datalib(mfiles, lib, gi, gu, galaxy_pass, galaxy_isa_upload_param, user_id, celery_obj)
    except error_perm as e:
        print('ERROR CATCH', e)
        if celery_obj:
            celery_obj.update_state(state='FAILURE',
                                    meta={'current': 0.0, 'total': 100, 'status': 'Failed {}'.format(e)})
        return 0
    except bioblend.ConnectionError as e:
        print('ERROR CATCH', e)
        if celery_obj:
            celery_obj.update_state(state='FAILURE',
                                    meta={'current': 0.0, 'total': 100, 'status': 'Failed {}'.format(e)})
        return 0



    return 1

def create_samplelist(user_id, igrp):
    dirpth = tempfile.mkdtemp()
    # dirpth = '/tmp/'
    nm = get_namemap()
    fnm = 'samplelist_{}.tabular'.format(igrp[0][nm['investigation']].replace(" ", "_"))
    tmp_pth = os.path.join(dirpth, fnm)

    with open(tmp_pth, 'w') as csvfile:
        writer = csv.writer(csvfile, delimiter='\t')
        writer.writerow(
                ['prefix', 'sample_class', 'code_field', 'blank', 'sample_type', 'full_path', 'original_filename', 'investigation',
                 'investigation_id', 'study', 'assay', 'fileid'])
        for m in igrp:
            if m[nm['sampletype']].lower() == 'blank':
                blank = 'yes'
            else:
                blank = 'no'

            code_field = m[nm['code_field']]
            code_field_r_compat = code_field.replace('_', '.')
            code_field_r_compat = code_field_r_compat.replace('-', '.')
            row = [
                m[nm['prefix']],
                code_field_r_compat,
                code_field,
                blank,
                m[nm['sampletype']],
                m['data_file'],
                m['original_filename'],
                m[nm['investigation']],
                m[nm['investigation_id']],
                m[nm['study']],
                m[nm['assay']],
                m['id']
            ]

            writer.writerow(row)
            investigation_id = m[nm['investigation_id']]


    misafile = MISAFile(investigation_id=investigation_id, user_id=user_id)
    misafile.original_filename = fnm
    misafile.data_file.save(fnm, File(open(tmp_pth)))
    misafile.save()

    fullpth = os.path.join(settings.MEDIA_ROOT, misafile.data_file.path)
    print(fullpth)
    return fullpth, misafile.id


def get_namemap():
    return  {'assay': 'run__assayrun__assaydetail__assay__name',
             'study': 'run__assayrun__assaydetail__assay__study__name',
             'investigation': 'run__assayrun__assaydetail__assay__study__investigation__name',
             'investigation_id': 'run__assayrun__assaydetail__assay__study__investigation__id',
             'sampletype': 'run__assayrun__assaydetail__studysample__sampletype__type',
             'code_field': 'run__assayrun__assaydetail__code_field',
             'prefix': 'run__prefix',
             }


def get_mfile_qs(pks):
    # get files associated with this isa project
    name_map = get_namemap()

    mfiles = MFile.objects.filter(
        run__assayrun__assaydetail__assay__study__investigation__id__in=pks
    ).values(
        name_map['assay'],
        name_map['study'],
        name_map['investigation'],
        name_map['investigation_id'],
        name_map['code_field'],
        name_map['sampletype'],
        name_map['prefix'],
        'data_file',
        'original_filename',
        'id'
    ).order_by(
        name_map['investigation'],
        name_map['study'],
        name_map['assay']
    )

    return mfiles



def create_isa_datalib(mfiles, lib, gi, gu, galaxy_pass, galaxy_isa_upload_param, user_id, celery_obj=''):
    name_map = get_namemap()
    igrps = group_by_keys(mfiles, (name_map['investigation'],))

    file_count = 0

    lc = LibraryClient(gi)
    fc = FoldersClient(gi)


    for igrp in igrps:
        print(igrp)
        # get the investigation name of the group, and create folder
        ifolder, sgrps = create_investigation_folder(igrp, lc, fc, lib, galaxy_isa_upload_param, name_map)
        samplelist_pth, misafile = create_samplelist(user_id, igrp)

        save_to_galaxy([samplelist_pth], galaxy_isa_upload_param, lc, gu, gi, galaxy_pass,
                                        lib['id'], ifolder['id'], 'samplelist', [{'id':misafile}])

        for sgrp in sgrps:
            print(sgrp)
            # get the study name of the group and create folder
            sfolder, agrps = create_study_folder(sgrp, lc, lib, name_map, ifolder)

            for agrp in agrps:
                print(agrp)

                study_n = agrp[0][name_map['study']]
                investigation_n = agrp[0][name_map['investigation']]
                assay_n = agrp[0][name_map['assay']]

                full_assay_name = '{}_{}_{}'.format(study_n, investigation_n, assay_n)

                if celery_obj:
                    if file_count==0:
                        count = 0.1
                    else:
                        count = file_count
                    celery_obj.update_state(state='RUNNING',
                                            meta={'current': count, 'total': len(mfiles)+1,
                                                  'status': 'Assay: {}'.format(assay_n)})

                afolder = create_assay_folder(agrp, lc, lib, name_map, sfolder)
                filelist = [os.path.join(settings.MEDIA_ROOT, f['data_file']) for f in agrp]

                data_lib_files = save_to_galaxy(filelist, galaxy_isa_upload_param, lc, gu, gi, galaxy_pass,
                                                 lib['id'], afolder['id'], full_assay_name, agrp)
                #
                file_count += len(data_lib_files)



def save_to_galaxy(filelist, galaxy_isa_upload_param, lc, gu, gi, galaxy_pass, lib_id, folder_id, full_name, filedetails):
    data_lib_files = add_filelist_datalib(filelist,
                                          galaxy_isa_upload_param,
                                          lc,
                                          gu,
                                          gi,
                                          galaxy_pass,
                                          lib_id,
                                          folder_id,
                                          full_name)

    link_files_in_galaxy(data_lib_files, filedetails, galaxy_isa_upload_param.galaxyinstancetracking, library=True)

    return data_lib_files


def create_investigation_folder(igrp, lc, fc, lib, galaxy_isa_upload_param, name_map):
    investigation_n = igrp[0][name_map['investigation']]
    print('========> Investigation:', investigation_n)

    if galaxy_isa_upload_param.remove:
        igt_for_removal = ISAGalaxyTrack.objects.filter(investigation__name=investigation_n)
        if igt_for_removal:
            for i in igt_for_removal:
                # remove folder
                fc.delete_folder(i.galaxy_id)
                # remove django entry
                i.delete()



    ifolder = lc.create_folder(library_id=lib['id'], description='isa_investigation', folder_name=investigation_n)[
        0]

    igt = ISAGalaxyTrack(galaxyinstancetracking=galaxy_isa_upload_param.galaxyinstancetracking,
                         isatogalaxyparam=galaxy_isa_upload_param,
                         investigation=Investigation.objects.get(name=investigation_n),
                         galaxy_id=ifolder['id']
                         )
    igt.save()
    sgrps = group_by_keys(igrp, (name_map['study'],))

    return ifolder, sgrps

def create_study_folder(sgrp, lc, lib, name_map, ifolder):
    study_n = sgrp[0][name_map['study']]
    print('====> Study:', study_n)
    sfolder = lc.create_folder(library_id=lib['id'],
                               folder_name=study_n,
                               description='isa_study',
                               base_folder_id=ifolder['id'])[0]

    agrps = group_by_keys(sgrp, (name_map['assay'],))
    return sfolder, agrps

def create_assay_folder(agrp, lc, lib, name_map, sfolder):
    # get the study name of the group and create folder
    assay_n = agrp[0][name_map['assay']]
    print('========> Assay:', assay_n)

    afolder = lc.create_folder(library_id=lib['id'],
                               folder_name=assay_n,
                               description='isa_assay',
                               base_folder_id=sfolder['id'])[0]

    return afolder









def group_by_keys(iterable, keys):
    # https://stackoverflow.com/questions/31955389/how-to-group-an-array-by-multiple-keys
    key_func = itemgetter(*keys)

    # For groupby() to do what we want, the iterable needs to be sorted
    # by the same key function that we're grouping by.
    sorted_iterable = sorted(iterable, key=key_func)

    return [list(group) for key, group in itertools.groupby(sorted_iterable, key_func)]


from django.conf.urls import url,include

from mogi import views

from rest_framework import routers
router = routers.DefaultRouter()
router.register(r'incoming_galaxy_data', views.IncomingGalaxyDataViewSet)

urlpatterns = [
    #############################################
    # General
    ##############################################

    url(r'^$', views.index, name='index'),
    url(r'^submitted/$', views.success, name='submitted'),
    url(r'^about/$', views.about, name='about'),

    #############################################
    # Galaxy ISA upload and workflows
    ##############################################
    url(r'^galaxy_isa_upload_datalib/$', views.GalaxyISAupload.as_view(), name='galaxy_isa_upload_datalib'),
    url(r'^isa_workflow_run/(?P<wid>\d+)$', views.ISAWorkflowRunView.as_view(), name='isa_workflow_run'),
    url(r'^isa_workflow_summary/', views.ISAWorkflowListView.as_view(), name='isa_workflow_summary'),
    url(r'^galaxy_isa_fileselect_upload_datalib', views.ISAFileSelectToGalaxyDataLib.as_view(), name='galaxy_isa_fileselect_upload_datalib'),
    url(r'^galaxy_isa_fileselect_upload_history', views.ISAFileSelectToGalaxyHist.as_view(), name='galaxy_isa_fileselect_upload_history'),

    #############################################
    # Galaxy ISA history data upload to django-metab
    ##############################################
    url(r'^history_mogi/$', views.HistoryMogiListView.as_view(), name='history_mogi'),
    url(r'^history_mogi_data/(?P<pk>\d+)/$', views.HistoryDataMogiListView.as_view(), name='history_mogi_data'),
    url(r'^history_mogi_data_save/(?P<history_internal_id>\d+)/(?P<galaxy_id>\w+)/$', views.HistoryDataMogiCreateView.as_view(), name='history_mogi_data_save'),

    #############################################
    # Peak and annotation summary
    ##############################################
    url(r'^cpeakgroupmeta_summary_mogi/$', views.CPeakGroupMetaListMogiView.as_view(), name='cpeakgroupmeta_summary_mogi'),
    url(r'^canns_all_mogi/$', views.CAnnotationListAllMogiView.as_view(), name='canns_all_mogi'),
    url(r'^canns_download_mogi/$', views.CAnnotationDownloadMogiView.as_view(), name='canns_download_mogi'),

    ##############################################
    # REST
    ##############################################
    url(r'^rest/', include(router.urls)),
    url(r'^incoming_galaxy_data_list/$', views.IncomingGalaxyDataListView.as_view(), name='incoming_galaxy_data_list'),
    url(r'^history_mogi_data_from_rest_save/(?P<galaxy_name>\w+)/(?P<galaxy_data_id>\w+)/(?P<galaxy_history_id>\w+)/$',
        views.HistoryDataMogiFromRestCreateView.as_view(), name='history_mogi_data_from_rest_save'),
    url(r'^save_lcms_from_from_rest/(?P<galaxy_name>[-\w]+)/(?P<galaxy_data_id>\w+)/(?P<galaxy_history_id>\w+)/(?P<investigation_name>[-\w]+)/$',
        views.SaveLcmsFromFromRest.as_view(), name='save_lcms_from_from_rest'),
]


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from misa.models import Investigation
from mogi.models import HistoryDataMOGI




class MisaGalaxyInteraction(TestCase):

    def test_investigation_link(self):

        # just check we can import misa models
        i = Investigation()
        i.save()

        self.assertEqual(len(Investigation.objects.all()), 1)

